package leodbc.javaagent.generation;

import javassist.CannotCompileException;
import javassist.CtBehavior;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import leodbc.javaagent.res.Constants;
import leodbc.javaagent.transformations.TransformerUtils;
import leodbc.support.LeoPost;

public class GeneratedPostConditionMethod extends GeneratedMethod 
{
	/**
	 * Generatore inizializzato per un certo metodo
	 * @param m
	 */
	public GeneratedPostConditionMethod(CtMethod m)
	{
		super(m);
	}
	

	/**
	 * Lista dei parametri generato per un metodo
	 * @param method Metodo dal quale estrarre la lista dei parametri
	 * @return Lista dei parametri
	 * @throws NotFoundException
	 */
	public String generatedMethodParameterList() throws NotFoundException {
		String methodParams;
		if(method.getReturnType().equals(CtClass.voidType))
		{
			methodParams="";
		} else
		{
			methodParams=method.getReturnType().getName()+" a";
		}
		return methodParams;
	}

	/**
	 * Tipologie dei parametri del metodo generato
	 * @param method Metodo da usare per la generazione
	 * @return Come da descrizione
	 * @throws NotFoundException
	 */
	public CtClass[] generatedMethodParameterTypes() throws NotFoundException 
	{
		CtClass[] methodTypes;
		if(method.getReturnType().equals(CtClass.voidType))
		{
			methodTypes=new CtClass[0];
		} else
		{
			methodTypes=new CtClass[1];
			methodTypes[0]=method.getReturnType();
		}
		return methodTypes;
	}
	
	/**
	 * @return Nome del metodo da generare
	 */
	@Override
	public String getGeneratedMethodName() throws NotFoundException
	{
		String sig=method.getSignature();
		sig.replace(',','_');
		sig=TransformerUtils.replace(sig, "(", "", true);
		sig=TransformerUtils.replace(sig, ")", "", true);
		sig=TransformerUtils.replace(sig, "[", "", true);
		sig=TransformerUtils.replace(sig, "]", "", true);
		sig=TransformerUtils.replace(sig, ".", "_", true);
		sig=TransformerUtils.replace(sig, "/", "_", true);
		sig=TransformerUtils.replace(sig, ";", "", true);
		return getMethodPrefix()+method.getName()+"_"+sig;
	}

	/**
	 * @return Nome del metodo da generare con nome di classe esplicito
	 */
	public String getGeneratedMethodNameExplicitClass(CtMethod met)
	{
		String sig=method.getSignature();
		sig.replace(',','_');
		sig=TransformerUtils.replace(sig, "(", "", true);
		sig=TransformerUtils.replace(sig, ")", "", true);
		sig=TransformerUtils.replace(sig, "[", "", true);
		sig=TransformerUtils.replace(sig, "]", "", true);
		sig=TransformerUtils.replace(sig, ".", "_", true);
		sig=TransformerUtils.replace(sig, "/", "_", true);
		sig=TransformerUtils.replace(sig, ";", "", true);
		return getMethodPrefix()+met.getName()+"_"+sig+"_"+met.getDeclaringClass().getName().replace('.', '_');
	}

	

	/**
	 * Invocazione alla superclasse
	 */
	public String getSuperclassMethodInvocation() throws NotFoundException
	{
		if(method.getReturnType().equals(CtClass.voidType))
		{
			return "super."+getGeneratedMethodName()+"();";
		} else
		{
			return "super."+getGeneratedMethodName()+"($$);";
		}
	}
	
	@Override
	/**
	 * @return Prefisso dei metodi da generare
	 */
	public String getMethodPrefix() {
		return Constants.POSTMETHOD_PREFIX;
	}


	@Override
	/**
	 * Gestisce un tag trovato in una interfaccia e ritorna
	 * la stringa del codice da generare per la sua gestione
	 * @param tag Tag trovato (il codice deve verificarne il tipo)
	 * @param clazz Classe che implemententa l'interfaccia
	 * @param method Metodo dove c' l'annotazione
	 * @return Codice di gestione
	 */
	public String handleTagInInterface(CtClass clazz, CtMethod method, Object tag) throws CannotCompileException, ClassNotFoundException, NotFoundException
	{
		if(tag instanceof LeoPost)
		{
			String mname=generateMethodFromInterfaceTag(clazz, (LeoPost)tag, method);
			if(method.getReturnType().equals(CtClass.voidType))
			{								
				return(mname+"();");
			} else
			{
				return(mname+"($1);");
			}
		}
		return "";
	}


	/**
	 * Inserisce nel metodo la chiamata al nuovo metodo generato
	 * @throws CannotCompileException
	 */
	public void injectMethodCall() throws NotFoundException, CannotCompileException {
		if(method.getReturnType().equals(CtClass.voidType))
		{
			method.insertAfter("{ "+getGeneratedMethodName()+"(); }");
		} else
		{
			method.insertAfter("{ "+getGeneratedMethodName()+"($_); }");
		}
	}

	/**
	 * Schema del metodo da generare
	 */
	public String getMethodSchema() {
		return Constants.POSTMETHOD_SCHEMA;
	}	
	
	/**
	 * Generazione del metodo delle postcondizioni
	 */
	public String generateMethodCode(CtBehavior method, Object ld, String superMethodInvocation) throws NotFoundException
	{
		String code=super.generateMethodCode(method, ld, superMethodInvocation);
		return code;
	}
}
