package leodbc.javaagent.generation;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import leodbc.javaagent.res.Constants;
import leodbc.support.LeoPre;

/**
 * Rappresenta un metodo generato per una precondizione
 * @author 2638966
 */
public class GeneratedPreConditionMethod extends GeneratedMethod 
{
	/**
	 * Generatore inizializzato per un certo metodo
	 * @param m
	 */
	public GeneratedPreConditionMethod(CtMethod m)
	{
		super(m);
	}

	/**
	 * Gestisce un tag trovato in una interfaccia e ritorna
	 * la stringa del codice da generare per la sua gestione
	 * @param tag Tag trovato (il codice deve verificarne il tipo)
	 * @param clazz Classe che implemententa l'interfaccia
	 * @param method Metodo dove c' l'annotazione
	 * @return Codice di gestione
	 */
	public String handleTagInInterface(CtClass clazz, CtMethod method, Object tag) throws CannotCompileException, ClassNotFoundException, NotFoundException
	{
		if(tag instanceof LeoPre)
		{
			String mname=generateMethodFromInterfaceTag(clazz, (LeoPre)tag, method);
			return(mname+"($$);");
		}
		return "";
	}	
	
	/**
	 * Lista dei parametri generato per un metodo
	 * @param method Metodo dal quale estrarre la lista dei parametri
	 * @return Lista dei parametri
	 * @throws NotFoundException
	 */
	public String generatedMethodParameterList() throws NotFoundException 
	{
		// Creazione della lista dei parametri
		String methodParams="";
		int i=1;
		for(CtClass type: method.getParameterTypes())
		{
			methodParams+=type.getName()+" a"+i+",";
			i++;
		}
		
		if(methodParams.length()>0) {
			methodParams=methodParams.substring(0, methodParams.length()-1);
		}
		return methodParams;
	}	

	/**
	 * Tipologie dei parametri del metodo generato
	 * @param method Metodo da usare per la generazione
	 * @return Come da descrizione
	 * @throws NotFoundException
	 */
	public CtClass[] generatedMethodParameterTypes() throws NotFoundException 
	{
		return method.getParameterTypes();
	}		
	
	/**
	 * @return Prefisso del metodo da generare
	 */
	public String getMethodPrefix()
	{
		return Constants.PREMETHOD_PREFIX;
	}
	
	/**
	 * @return Genera l'invocazione al metodo della classe padre
	 */
	public String getSuperclassMethodInvocation() throws NotFoundException
	{
		return "super."+super.getGeneratedMethodName()+"($$);";
	}

	/**
	 * Inserisce nel metodo la chiamata al nuovo metodo generato
	 * @throws CannotCompileException
	 */
	public  void injectMethodCall() throws CannotCompileException, NotFoundException
	{
		method.insertBefore("{ "+getGeneratedMethodName()+"($$); }");
	}


	/**
	 * @return Schema del metodo da generare
	 */
	public String getMethodSchema() 
	{
		return Constants.PREMETHOD_SCHEMA;
	}	
}
