package leodbc.javaagent.instrumentor;

import java.util.HashMap;
import java.util.Map;

import javassist.ClassPool;
import javassist.LoaderClassPath;

/**
 * Classe che gestisce l'associazione fra i ClassLoader ed i ClassPool
 * 
 * @author Leonardo Cecchi
 */
public class ClassLoaderToClassPool 
{
	/**
	 * Tabella che mappa i ClassLoader ai ClassPool
	 */
	private Map<ClassLoader, ClassPool> classLoaderToClassPool;
	
	/**
	 * Inizializzazione
	 */
	public ClassLoaderToClassPool()
	{
		classLoaderToClassPool=new HashMap<ClassLoader, ClassPool>();
	}

	/**
	 * Ritorna il ClassPool da usare per un certo class loader
	 * @param cl ClassLoader da usare
	 * @return ClassPool
	 */
	public ClassPool getClassPoolForClassLoader(ClassLoader cl)
	{
		if(classLoaderToClassPool.containsKey(cl))
		{
			return classLoaderToClassPool.get(cl);
		} else
		{
			ClassPool cp;
			if(cl==null)
			{
				cp=ClassPool.getDefault();
			} else
			{
				cp=new ClassPool();
				cp.insertClassPath(new LoaderClassPath(cl));
			}
			classLoaderToClassPool.put(cl, cp);
			return cp;
		}
	}	
}
