package leodbc.javaagent.transformations;

import java.util.LinkedList;

import javassist.CannotCompileException;
import javassist.CtBehavior;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.NotFoundException;
import leodbc.javaagent.res.Constants;
import leodbc.support.LeoArgCheck;

public class ArgCheckTransformer extends MethodTransformer 
{
	@Override
	/**
	 * Gestione di un costruttore
	 */
	public void handleConstructor(CtConstructor method)
			throws ClassNotFoundException, CannotCompileException,
			NotFoundException {
		String code=handleBehaviour(method);
		if(code.length()!=0)
		{
			method.insertBeforeBody(code);
		}
	}

	/**
	 * Gestisce un metodo ed un costruttore
	 * @param method
	 * @return Il codice da generare
	 */
	public String handleBehaviour(CtBehavior method) throws ClassNotFoundException,
		CannotCompileException, NotFoundException 
	{
		LeoArgCheck lp=null;
		LinkedList<String> conditions=new LinkedList<String>();
		for(Object[] paramType: method.getParameterAnnotations())
		{
			for(Object an: paramType)
			{
				if(an instanceof LeoArgCheck)
				{
					lp=(LeoArgCheck)an;
					checkAnnotation(lp);
					conditions.add(lp.value());
				}
			}
		}
		if(conditions.size()!=0)
		{
			String conjunctiveCondition="";
			for(String cond:conditions)
			{
				conjunctiveCondition+=cond+" && ";
			}
			conjunctiveCondition=conjunctiveCondition.substring(0,conjunctiveCondition.length()-4);
			String code=TransformerUtils.replace(Constants.CHECK_ILLEGAL_ARGUMENT, 
					Constants.CONDITION_MARKER, conjunctiveCondition, true);
			return code;
		}
		return "";		
	}
	
	/**
	 * Gestione di un metodo
	 */
	@Override
	public void handleMethod(CtMethod method) throws ClassNotFoundException,
			CannotCompileException, NotFoundException {
		String code=handleBehaviour(method);
		if(code.length()!=0)
		{
			method.insertBefore(code);
		}
	}

}
