package leodbc.javaagent.transformations;

import java.lang.reflect.Modifier;

import leodbc.javaagent.res.Constants;
import leodbc.support.LeoInvariant;
import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.CtNewMethod;
import javassist.NotFoundException;

/**
 * Trasformatore di classe per le precondizioni
 * @author Leonardo Cecchi
 */
public class InvariantTransformer implements ClassTransformer
{
	/**
	 * Trasformatore per le invarianti
	 */
	public void transform(CtClass clazz) throws CannotCompileException, ClassNotFoundException, NotFoundException 
	{
		LeoInvariant inv=null;
		for(Object o: clazz.getAnnotations())
		{
			if(o instanceof LeoInvariant)
			{
				inv=(LeoInvariant)o;
			}
		}

		if(inv!=null)
		{
			if(inv.value().indexOf(Constants.CONDITION_MARKER)!=-1 || inv.value().indexOf(Constants.LAUNCH_SUPERCLASS_MARKER)!=-1)
			{
				throw new CannotCompileException("Wrong precondition: "+inv.value()+" it have \""+Constants.CONDITION_MARKER+"\" inside!");
			}
		}
		
		// Creazione del codice del metodo
		String codiceMetodo=Constants.INVARIANT_METHOD;
		
		// Richiamo alla superclasse
		if(TransformerUtils.classHasMethod(clazz.getSuperclass(), Constants.INVARIANT_METHOD_NAME, new CtClass[0]))
		{
			codiceMetodo=TransformerUtils.replace(codiceMetodo, Constants.LAUNCH_SUPERCLASS_MARKER, 
					Constants.INVARIANT_METHOD_NAME+"();", true);
		} else
		{
			codiceMetodo=TransformerUtils.replace(codiceMetodo, Constants.LAUNCH_SUPERCLASS_MARKER, 
					"", true);
		}
		
		// Invarianti nelle interfacce
		String invariantiInterfacce="";
		boolean hasInvariantInterface=false;
		for(CtClass k:clazz.getInterfaces())
		{
			for(Object annotation: k.getAnnotations())
			{
				if (annotation instanceof LeoInvariant)
				{
					LeoInvariant li=(LeoInvariant) annotation;
					invariantiInterfacce+=li.value()+" &&";
					hasInvariantInterface=true;
				}
			}
		}
		
		String condizioni=inv!=null?inv.value():"true";
		if(invariantiInterfacce.length()>0)
		{
			condizioni+=" && "+invariantiInterfacce.substring(0, invariantiInterfacce.length()-3);
		}
		
		
		// Installazione della condizione
		codiceMetodo=TransformerUtils.replace(codiceMetodo, Constants.CONDITION_MARKER, condizioni, true);
		
		if(inv!=null || hasInvariantInterface)
		{
			// Debugging
			if(Constants.INTERNAL_DEBUGGING)
			{
				System.out.println(clazz.getName()+": "+codiceMetodo);
			}
			
			// Via
			clazz.addMethod(CtNewMethod.make(codiceMetodo, clazz));
	
			// Aggiunta invarianti ai metodi
			for(CtMethod method: clazz.getDeclaredMethods())
			{
				if(!Modifier.isStatic(method.getModifiers()) && !method.getName().equals(Constants.INVARIANT_METHOD_NAME))
				{
					// Aggiunta del check di invariante
					method.insertAfter("{"+Constants.INVARIANT_METHOD_NAME+"(); }");
				}
			} 
			
			// Aggiunta invarianti ai costruttori
			for(CtConstructor con: clazz.getDeclaredConstructors())
			{
				con.insertAfter("{"+Constants.INVARIANT_METHOD_NAME+"(); }");
			}
		}
	}
}
