package leodbc.javaagent.transformations;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.NotFoundException;
import leodbc.javaagent.res.Constants;

/**
 * Funzionalit comuni alla trasformazione di metodi e di costruttori
 * 
 * @author 2638966
 */
public abstract class MethodTransformer implements ClassTransformer
{
	/**
	 * Controllo di validit della annotazione
	 * @param ld Annotazione da controllare
	 * @throws CannotCompileException
	 */
	public void checkAnnotation(Object ld) throws CannotCompileException 
	{
		if(ld!=null)
		{
			String assertion=TransformerUtils.extractValueFromAnnotation(ld);
			if(assertion.indexOf(Constants.CONDITION_MARKER)!=-1 || assertion.indexOf(Constants.LAUNCH_SUPERCLASS_MARKER)!=-1)
			{
				throw new CannotCompileException("Wrong precondition: "+assertion+" it have \""+Constants.CONDITION_MARKER+"\" inside!");
			}
		}
	}

	/**
	 * Gestione di un costruttore.
	 * Viene chiamato ad ogni costruttore della classe
	 * @param method Metodo da gestire
	 * @throws ClassNotFoundException
	 * @throws CannotCompileException
	 * @throws NotFoundException
	 */
	public abstract void handleConstructor(CtConstructor method) throws ClassNotFoundException, CannotCompileException, NotFoundException;
	
	/**
	 * Gestione di un metodo.
	 * Viene chiamato ad ogni metodo della classe
	 * @param method Metodo da gestire
	 * @throws ClassNotFoundException
	 * @throws CannotCompileException
	 * @throws NotFoundException
	 */
	public abstract void handleMethod(CtMethod method) throws ClassNotFoundException, CannotCompileException, NotFoundException;
	
	/**
	 * Trasformatore generico
	 */
	public void transform(CtClass clazz) throws CannotCompileException, ClassNotFoundException, NotFoundException 
	{
		for(CtMethod method: clazz.getMethods())
		{
			if(!method.getName().startsWith(Constants.POSTMETHOD_PREFIX) &&
					!method.getName().startsWith(Constants.PREMETHOD_PREFIX) &&
					!method.getName().equals(Constants.INVARIANT_METHOD_NAME))
			{
				handleMethod(method);
			}
		}
		for(CtConstructor cons: clazz.getConstructors())
		{
			handleConstructor(cons);
		}
	}
}
