package leodbc.javaagent.transformations;

import leodbc.javaagent.generation.GeneratedMethod;
import leodbc.javaagent.generation.GeneratedPreConditionMethod;
import leodbc.javaagent.res.Constants;
import leodbc.support.LeoPre;
import javassist.CannotCompileException;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.NotFoundException;

/**
 * Trasformatore di classe per le precondizioni
 * @author Leonardo Cecchi
 */
public class PreconditionTransformer extends MethodTransformer implements ClassTransformer
{	
	/**
	 * Gestione di un metodo
	 * @param method Metodo da gestire
	 * @throws ClassNotFoundException
	 * @throws CannotCompileException
	 * @throws NotFoundException
	 */
	public void handleMethod(CtMethod method) throws ClassNotFoundException, CannotCompileException, NotFoundException {
		LeoPre lp=null;
		for(Object an: method.getAnnotations())
		{
			if(an instanceof LeoPre)
			{
				lp=(LeoPre)an;
				checkAnnotation(lp);
			}
		}
		GeneratedMethod gen=new GeneratedPreConditionMethod(method);
		gen.handleTag(lp);
	}
	
	/**
	 * Ritorna il codice di controllo di una precondizione in un costruttore
	 */
	public String generateConstructorPreconditionCode(String assertions)
	{
		return TransformerUtils.replace(Constants.CHECK_PRECONDITION_CONSTRUCTOR, 
				Constants.CONDITION_MARKER, assertions, true);
	}
	
	/**
	 * Gestione di un costruttore.
	 * Viene chiamato ad ogni costruttore della classe
	 * @param method Metodo da gestire
	 * @throws ClassNotFoundException
	 * @throws CannotCompileException
	 * @throws NotFoundException
	 */
	public void handleConstructor(CtConstructor method) throws ClassNotFoundException, CannotCompileException, NotFoundException
	{
		LeoPre lp=null;
		for(Object an: method.getAnnotations())
		{
			if(an instanceof LeoPre)
			{
				lp=(LeoPre)an;
				checkAnnotation(lp);
				method.insertBeforeBody(generateConstructorPreconditionCode(lp.value()));
			}
		}
	}
}
