package leodbc.javaagent.transformations;

import javassist.CtClass;
import javassist.NotFoundException;
import leodbc.support.LeoArgCheck;
import leodbc.support.LeoPost;
import leodbc.support.LeoPre;

/**
 * Classe di utilit per i trasformatori
 * 
 * @author Leonardo Cecchi
 */
public class TransformerUtils 
{
	/**
	 * @param myClass Classe dove fare il check
	 * @param methodName Nome del metodo da cercare
	 * @param types Tipi dei parametri
	 * @return True se l'ha trovato, false altrimenti
	 */
	public static boolean classHasMethod(CtClass myClass, String methodName, CtClass[] types) throws NotFoundException
	{
		if(myClass==null)
		{
			return false;
		}
		
		try
		{
			myClass.getDeclaredMethod(methodName, types);
			return true;
		} catch(NotFoundException nfe)
		{
			return classHasMethod(myClass.getSuperclass(), methodName, types);
		}
	}
	
	/**
	 * Replace all occurrences of o in str with n, 
	 * or only the first occurrence if all is false. 
	 * replace("aaaa", "aa", "bbb", false) returns "bbbaa" 
	 * replace("aaaa", "aa", "bbb", true)  returns "bbbbbb"
	 * 
	 * @param str Stringa
	 * @param o Da cercare
	 * @param n Cosa metterci
	 * @param all Tutte le occorrenze?
	 * @return Il risultato
	 */ 
	public static String replace(String str, String o, String n, boolean all) { 
	    if (str == null || o == null || o.length() == 0 || n == null) 
	        throw new IllegalArgumentException("null or empty String"); 
	    StringBuffer result = null; 
	    int oldpos = 0; 
	    do { 
	        int pos = str.indexOf(o, oldpos); 
	        if (pos < 0) 
	            break; 
	        if (result == null) 
	            result = new StringBuffer(); 
	        result.append(str.substring(oldpos, pos)); 
	        result.append(n); 
	        pos += o.length(); 
	        oldpos = pos; 
	    } while (all); 
	    if (oldpos == 0) { 
	        return str; 
	    } else { 
	        result.append(str.substring(oldpos)); 
	        return new String(result); 
	    } 
	} 	
	
	/**
	 * Estrae la asserzione da una annotazione conosciuta
	 * @param ld Asserzione
	 * @return valore
	 */
	public static String extractValueFromAnnotation(Object ld) 
	{
		String assertions="";
		if(ld==null)
		{
			assertions="";
		} else if(ld instanceof LeoPre)
		{
			assertions=((LeoPre)ld).value();
		} else if(ld instanceof LeoPost)
		{
			assertions=((LeoPost)ld).value();
		} else if(ld instanceof LeoArgCheck)
		{
			assertions=((LeoArgCheck)ld).value();
		}else
		{
			throw new AssertionError("Tipo di tag non conosciuto:"+ld.getClass().toString());
		}
		return assertions;
	}	
}
