package leodbc.support;

import java.util.Stack;

/**
 * Classe che memorizza i vecchi valori delle variabili
 * @author Leonardo Cecchi
 */
public class StackVariableStore extends AbstractVariableStore
{
	// Memorizzazione dei valori
	private Stack<NameValueMapping> dataStore;
	// Linea di separazione
	private final static String SEPARATION_LINE_MARKER="--SeparationLine--";
	
	/**
	 * Nuova instanza vuota
	 */
	public StackVariableStore()
	{
		dataStore=new Stack<NameValueMapping>();
	}
	
	/* (non-Javadoc)
	 * @see leodbc.support.VariableStore#putValue(java.lang.String, java.lang.Object)
	 */
	public boolean putValue(String name, Object val)
	{
		NameValueMapping nvm=new NameValueMapping(name,val);
		dataStore.push(nvm);
		return true;
	}
	
	/**
	 * Linea di separazione dello stack per un metodo
	 */
	public void addMethodMarker()
	{
		if(dataStore.size()==0 || !dataStore.peek().getName().equals(SEPARATION_LINE_MARKER))
		{
			dataStore.push(new NameValueMapping(SEPARATION_LINE_MARKER,null));
		}
	}
	
	/** 
	 * Cerca un elemento nello stack per il metodo corrente (fino alla linea di
	 * separazione)
	 * @see leodbc.support.VariableStore#getValue(java.lang.String)
	 */
	public Object getValue(String name)
	{
		Object result=null;
		for(int i=dataStore.size()-1;i>=0;i--)
		{
			NameValueMapping nvm=dataStore.get(i);
			if(nvm.getName().equals(SEPARATION_LINE_MARKER))
			{
				// Non trovato
				break;
			} else if(nvm.getName().equals(name))
			{
				result=nvm.getValue();
				break;
			}
		}
		return result;
	}
	
	/**
	 * Rimuove le associazioni di un metodo (ovvero tutto fino alla separation
	 * line compresa)
	 */
	public void removeMethodAssociations()
	{
		while(!dataStore.isEmpty())
		{
			NameValueMapping nvm=dataStore.peek();
			dataStore.pop();
			if(nvm.getName().equals(SEPARATION_LINE_MARKER))
			{
				break;
			} 
		}
	}
}
