package leodbc.support;

import java.util.HashMap;

public class ThreadSensitiveVariableStore extends AbstractVariableStore
{
	private HashMap<Long, VariableStore> dataStore;
	
	/**
	 * Memorizzatore di variabili che alloca una instanza per thread
	 *
	 */
	public ThreadSensitiveVariableStore()
	{
		dataStore=new HashMap<Long, VariableStore>();
	}
	
	/**
	 * Imposta un valore
	 * @param name Nome
	 * @param val Valore
	 */
	public boolean putValue(String name, Object val)
	{
		createStoreForThread();
		return dataStore.get(Thread.currentThread().getId()).putValue(name, val);
	}

	private void createStoreForThread() {
		if(!dataStore.containsKey(Thread.currentThread().getId()))
		{
			dataStore.put(Thread.currentThread().getId(), new StackVariableStore());
		}
	}

	/**
	 * Riprende un valore
	 * @return il valore memorizzato oppure null se non presente
	 */
	public Object getValue(String name)
	{
		if(!dataStore.containsKey(Thread.currentThread().getId()))
		{
			return null;
		} else
		{
			return dataStore.get(Thread.currentThread().getId()).getValue(name);
		}
	}

	/**
	 * Linea di separazione dello stack pe run metodo
	 */
	public void addMethodMarker()
	{
		createStoreForThread();
		dataStore.get(Thread.currentThread().getId()).addMethodMarker();
	}

	/**
	 * Rimuove le associazioni di un metodo (ovvero tutto fino alla separation
	 * line compresa)
	 */
	public void removeMethodAssociations()
	{
		createStoreForThread();
		dataStore.get(Thread.currentThread().getId()).removeMethodAssociations();
	}
}
